/*
 * Linea.java
 *
 * Created on 30 de abril de 2007, 11:19 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.JPanel.*;
import javax.swing.JLabel.*;
import java.awt.GridLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JProgressBar;
import java.awt.FlowLayout;
import javax.swing.BoxLayout;
import javax.swing.BoxLayout.*;
/**
 * Esta es la clase abstracta base para todas las componentes, tiene definido me-
 * todos no abstractos y abstractos.
 * Todas las componentes debieran sobreescribir los metodos: dibujate, contiene,
 * rotar, mover y clonar.
 * @author Francisco Olivares
 * @author Cristhoper Jana
 */
public abstract class Linea implements Cloneable {
    /** Color de la componente
     */
    private Color color;
    /**
     * Es tanto para el punto de mas arriba, como para el punto de mas a la izquierda.
     * La idea es que rotar tenga dos estados para las componentes electricas, siempre
     * deje este punto como el de arriba y el de la izquierda.
     */
    protected Point arribaIzquierda;
    /**
     * Es tanto para el punto de mas abajo, como para el punto de mas a la derecha.
     * La idea es que rotar tenga dos estados para las componentes electricas, siempre
     * deje este punto como el de abajo y el de la derecha.
     */
    protected Point abajoDerecha;
    private final int LOCACIONPORDEFECTO  = 50;
    /**
     * Las figuras para reconocer que estan siendo apuntadas por el mouse utilizan
     * el metodo, es una medida de la sensibilidad que va a tener la comonente
     * al acercarse el mouse, antes de cambiar de color.
     */
    protected final int ANCHORECTANGULOCAMBIOCOLOR = 2;
    /**
     * Esta variable de instancia representa el largo que tienen las componentes,
     * este valor no debe ser sobreescrito por ninguno de los componentes, ya que
     * si no no van a calzar bien en el grafico.
     */
    protected final int LARGODELASFIGURAS = 90;
    /** Creates a new instance of Linea */
    public Linea() {
        color = Color.BLACK;
        arribaIzquierda = new Point(LOCACIONPORDEFECTO,LOCACIONPORDEFECTO);
        abajoDerecha    = new Point(LOCACIONPORDEFECTO,LOCACIONPORDEFECTO + LARGODELASFIGURAS);
    }
    /**
     * Segundo constructor, el cual toma dos enteros para situar la figura,
     * el segundo punto es fijado abajo del dado, considerandose para esto el
     * tamagno de la componente.
     * @param x Representa el punto x de el punto de mas arrbiba a la izquierda que
     * tiene la componente.
     * @param y Representa el punto y de el punto de mas arrbiba a la izquierda que
     * tiene la componente.
     */
    public Linea(int x, int y) {
        color = Color.BLACK;
        arribaIzquierda = new Point(x,y);
        abajoDerecha    = new Point(x,y + LARGODELASFIGURAS);
    }
    /**
     * Retorna el valor en X del punto superior, si la figura esta en posicion vertical.
     * Si se encuentra en posicion horizontal, retorna el valor en X de la izquierda.
     */
    public int getArribaIzquierdaX() {
        return(arribaIzquierda.x);
    }
    /**
     * Retorna el valor en Y del punto superior, si la figura esta en posicion vertical.
     * Si se encuentra en posicion horizontal, retorna el valor en Y de la izquierda.
     */
    public int getArribaIzquierdaY() {
        return(arribaIzquierda.y);
    }
    /**
     * Retorna el valor en X del punto inferior, si la figura esta en posicion vertical.
     * Si se encuentra en posicion horizontal, retorna el valor en X de la derecha.
     */
    public int getAbajoDerechaX() {
        return(abajoDerecha.x);
    }
     /**
     * Retorna el valor en Y del punto inferior, si la figura esta en posicion vertical.
     * Si se encuentra en posicion horizontal, retorna el valor en Y de la derecha.
     */
    public int getAbajoDerechaY() {
        return(abajoDerecha.y);
    }
    /**
     * Retorna el valor del punto superior, si la figura esta en posicion vertical.
     * Si se encuentra en posicion horizontal, retorna el valor de la izquierda.
     * @return Entrega el Punto superior o izquierdo
     */
    public Point getArribaIzquierda() {
        return(arribaIzquierda);
    }
    /**
     * Retorna el valor del punto inferior, si la figura esta en posicion vertical.
     * Si se encuentra en posicion horizontal, retorna el valor de la derecha.
     * @return Entrega el Punto inferior o derecho.
     */
    public Point getAbajoDerecha() {
        return(abajoDerecha);
    }
    /**
     * Setea el punto de mas arriba a la izquierda
     * @param x Es la absisa por la cual se va a cambiar la posicion del punto
     * de mas arriba a la izquierda.
     * @param y Es la ordenada por la cual se va a cambiar la posicion del punto
     * de mas arriba a la izquierda.
     */
    public void setArribaIzquierda(int x, int y) {
        arribaIzquierda.setLocation(x,y);
    }
    /**
     * Setea el punto de mas abajo a la derecha
     * @param x Es la absisa por la cual se va a cambiar la posicion del punto
     * de mas abajo a la derecha.
     * @param y Es la ordenada por la cual se va a cambiar la posicion del punto
     * de mas abajo a la derecha.
     */
    public void setAbajoDerecha(int x, int y) {
        abajoDerecha.setLocation(x,y);
    }
    /**
     * Este metodo me permite cambiarle el color a las componentes.
     * @param color Este es el color por el cual la componente va a cambiar.
     */
    public void cambioDeColor(Color color) {
        this.color = color;
    }
    /**
     * Este metodo me entrega el color actual de la componente.
     * No sobreescribir.
     * @return Retorna un color, que es el color de la componente.
     */
    public Color getColor() {
        return(color);
    }
    /** Metodo que implementa el clone de la clase Lineas.
     *  TODAS LAS COMPONENTES DEBEN SOBREESCRIBIR ESTE METODO Y IMPLEMENTAR
     *  LA INTERFAS CLONEABLE, ESTE METODO ES MUY GENERICO, Y SI FALLA VA A
     *  BOTAR LA APLICACION, POR LO QUE DEBE SER SOBRESCRITO.
     *  ESTE METODO ES UTILIZADO AL SELECCIONAR COMPONENTES Y MOVERLAS POR
     *  LA PANTALLA.
     *  @return Objeto que debe ser casteado (identico al entregado),
     *  tipo Object.
     */
    public Object clone() {
        Object obj = null;
        try{
            obj = super.clone();
        }catch(CloneNotSupportedException ex) {
            System.exit(0);
        }
        return(obj);
    }
    /**
     * Este metodo define el movimiento de las figuras, como una variacion de la
     * posicion anterior. Puede que no sea necesario sobreescribir este metodo.
     */
    public void mover(int x, int y) {
        double deltaX = arribaIzquierda.getX() + x;
        double deltaY = arribaIzquierda.getY() + y;
        arribaIzquierda.setLocation(deltaX, deltaY);

        deltaX = abajoDerecha.getX() + x;
        deltaY = abajoDerecha.getY() + y;
        abajoDerecha.setLocation(deltaX, deltaY);
    }
    /**
     * Las compoentes deben sobreescribir este metodo, todas las componentes se
     * definen por dos puntos, inclusive la tierra, ya que es necesario para saber
     * hacia donde esta orientada, (aunque podria ser que la tierra no fuera
     * rotada).
     * La idea es que las componentes tengan dos estados de rotacion,
     * horizontal y vertical, y que al rotar se intercambie entre estos estados
     * por lo que un punto quedaria fijo que seria el de mas arriba, definido de
     * esta forma, y el otro punto cambie estando o abajo, como a la derecha
     * de este punto.
     */
    public abstract void rotar();
    /**
     * Este metodo dibuja el tipo de figura, es de tipo graphics simple,
     * ya que no solo se dibujaran componentes tipo lineas, que no
     * requieren graficos muy complicados.
     * @return retorna el mismo grafico entregado, incorporando la
     * componente que define la clase.
     * @param g Elemento grafico al cual se le va a agregar las componentes electricas.
     */
    public abstract Graphics dibujate(Graphics g);
    /**
     * Metodo que utiliza la constante ANCHORECTANGULOCAMBIOCOLOR como ancho que tiene
     * el rectangulo que contiene a la figura que se representa graficamente,
     * es decir si fuera una linea, este metodo verificaria si el mouse esta
     * contenido dentro de un rectangulo de ancho 2*ANCHORECTANGULOCAMBIOCOLOR, y el
     * largo igual a 2*ANCHORECTANGULOCAMBIOCOLOR + largoDeLaLinea().
     * @param x Este es el punto x de donde apunta el mouse para probar si el
     * punto esta contenido en la Componente.
     * @param y Este es el punto y de donde apunta el mouse para probar si el
     * punto esta contenido en la Componente.
     * @return Retorna verdadero si la figura el punto esta contenido,
     * y falso si no esta contenido.
     */
    public abstract boolean contiene(int x, int y);
}
